@@ -20,6 +20,7 @@
 #include "postgres.h"
 
 #include "catalog/dependency.h"
+#include "catalog/namespace.h"
 #include "catalog/objectaccess.h"
 #include "catalog/pg_class_d.h"
 #include "catalog/pg_namespace_d.h"
@@ -103,10 +104,10 @@ void ag_ProcessUtility_hook(PlannedStmt *pstmt, const char *queryString, bool re
 
 static void drop_age_extension(DropStmt *stmt)
 {
-    // Remove all graphs
+    /* Remove all graphs */
     drop_graphs(get_graphnames());
 
-    // Remove the object access hook
+    /* Remove the object access hook */
     object_access_hook_fini();
 
     /*
@@ -120,7 +121,7 @@ static void drop_age_extension(DropStmt *stmt)
     clear_global_Oids_GRAPHID();
 }
 
-// Check to see if the Utility Command is to drop the AGE Extension.
+/* Check to see if the Utility Command is to drop the AGE Extension. */
 static bool is_age_drop(PlannedStmt *pstmt)
 {
     ListCell *lc;
@@ -162,10 +163,17 @@ static void object_access(ObjectAccessType access, Oid class_id, Oid object_id,
     if (prev_object_access_hook)
         prev_object_access_hook(access, class_id, object_id, sub_id, arg);
 
-    // We are interested in DROP SCHEMA and DROP TABLE commands.
+    /* We are interested in DROP SCHEMA and DROP TABLE commands. */
     if (access != OAT_DROP)
         return;
 
+    /*
+     * Age might be installed into shared_preload_libraries before extension is
+     * created. In this case we must bail out from this hook.
+     */
+    if (!OidIsValid(get_namespace_oid("ag_catalog", true)))
+        return;
+
     drop_arg = arg;
 
     /*
@@ -204,7 +212,7 @@ static void object_access(ObjectAccessType access, Oid class_id, Oid object_id,
 
         cache_data = search_label_relation_cache(object_id);
 
-        // We are interested in only tables that are labels.
+        /* We are interested in only tables that are labels. */
         if (!cache_data)
             return;
 
