@@ -19,9 +19,9 @@
 
 #include "postgres.h"
 
-#include "access/xact.h"
 #include "access/genam.h"
 #include "access/heapam.h"
+#include "access/xact.h"
 #include "catalog/dependency.h"
 #include "catalog/objectaddress.h"
 #include "commands/defrem.h"
@@ -35,14 +35,14 @@
 #include "nodes/pg_list.h"
 #include "nodes/value.h"
 #include "parser/parser.h"
-#include "utils/fmgroids.h"
-#include "utils/relcache.h"
 #include "utils/rel.h"
+#include "utils/relcache.h"
 
 #include "catalog/ag_graph.h"
 #include "catalog/ag_label.h"
 #include "commands/label_commands.h"
 #include "utils/graphid.h"
+#include "commands/graph_commands.h"
 #include "utils/name_validation.h"
 
 /*
@@ -61,31 +61,45 @@ PG_FUNCTION_INFO_V1(create_graph);
 /* function that is evoked for creating a graph */
 Datum create_graph(PG_FUNCTION_ARGS)
 {
-    char *graph;
     Name graph_name;
-    char *graph_name_str;
-    Oid nsp_id;
 
-    //if no argument is passed with the function, graph name cannot be null
+    /* if no argument is passed with the function, graph name cannot be null */
     if (PG_ARGISNULL(0))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("graph name can not be NULL")));
     }
 
-    //gets graph name as function argument
+    /* gets graph name as function argument */
     graph_name = PG_GETARG_NAME(0);  
 
+    create_graph_internal(graph_name);
+
+    ereport(NOTICE,
+            (errmsg("graph \"%s\" has been created", NameStr(*graph_name))));
+
+    /* 
+     * According to postgres specification of c-language functions
+     * if function returns void this is the syntax.
+     */
+    PG_RETURN_VOID(); 
+}
+
+Oid create_graph_internal(const Name graph_name)
+{
+    Oid nsp_id;
+    char *graph_name_str;
+
     graph_name_str = NameStr(*graph_name);
 
-    //checking if the name of the graph falls under the pre-decided graph naming conventions(regex)
+    /* checking if the name of the graph falls under the pre-decided graph naming conventions(regex) */
     if (!is_valid_graph_name(graph_name_str))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("graph name is invalid")));
     }
 
-    //graph name must be unique, a graph with the same name should not exist
+    /* graph name must be unique, a graph with the same name should not exist */
     if (graph_exists(graph_name_str))
     {
         ereport(ERROR,
@@ -95,22 +109,44 @@ Datum create_graph(PG_FUNCTION_ARGS)
 
     nsp_id = create_schema_for_graph(graph_name);
 
-    //inserts the graph info into the relation which has all the other existing graphs info
+    /* inserts the graph info into the relation which has all the other existing graphs info */
     insert_graph(graph_name, nsp_id);  
 
-    //Increment the Command counter before create the generic labels.
+    /* Increment the Command counter before create the generic labels. */
     CommandCounterIncrement();
 
-    //Create the default label tables
-    graph = graph_name->data;
-    create_label(graph, AG_DEFAULT_LABEL_VERTEX, LABEL_TYPE_VERTEX, NIL);
-    create_label(graph, AG_DEFAULT_LABEL_EDGE, LABEL_TYPE_EDGE, NIL);
+    /* Create the default label tables */
+    create_label(graph_name_str, AG_DEFAULT_LABEL_VERTEX, LABEL_TYPE_VERTEX, NIL);
+    create_label(graph_name_str, AG_DEFAULT_LABEL_EDGE, LABEL_TYPE_EDGE, NIL);
 
-    ereport(NOTICE,
-            (errmsg("graph \"%s\" has been created", NameStr(*graph_name))));
+    return nsp_id;
+}
 
-    //according to postgres specification of c-language functions if function returns void this is the syntax
-    PG_RETURN_VOID(); 
+PG_FUNCTION_INFO_V1(age_graph_exists);
+
+Datum age_graph_exists(PG_FUNCTION_ARGS)
+{
+    Name graph_name;
+    char *graph_name_str;
+
+    /* if no argument is passed with the function, graph name cannot be null */
+    if (PG_ARGISNULL(0))
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                        errmsg("graph name can not be NULL")));
+    }
+
+    graph_name = PG_GETARG_NAME(0);
+    graph_name_str = NameStr(*graph_name);
+
+    if (graph_exists(graph_name_str))
+    {
+        return boolean_to_agtype(true);
+    }
+    else
+    {
+        return boolean_to_agtype(false);
+    }
 }
 
 static Oid create_schema_for_graph(const Name graph_name)
@@ -158,7 +194,7 @@ static Oid create_schema_for_graph(const Name graph_name)
     schema_stmt->if_not_exists = false;
     nsp_id = CreateSchemaCommand(schema_stmt,
                                  "(generated CREATE SCHEMA command)", -1, -1);
-    // CommandCounterIncrement() is called in CreateSchemaCommand()
+    /* CommandCounterIncrement() is called in CreateSchemaCommand() */
 
     return nsp_id;
 }
@@ -182,9 +218,8 @@ Datum drop_graph(PG_FUNCTION_ARGS)
     graph_name_str = NameStr(*graph_name);
     if (!graph_exists(graph_name_str))
     {
-        ereport(ERROR,
-                (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                 errmsg("graph \"%s\" does not exist", graph_name_str)));
+        ereport(ERROR, (errcode(ERRCODE_UNDEFINED_SCHEMA),
+                        errmsg("graph \"%s\" does not exist", graph_name_str)));
     }
 
     drop_schema_for_graph(graph_name_str, cascade);
@@ -209,7 +244,7 @@ static void drop_schema_for_graph(char *graph_name_str, const bool cascade)
      * so the event triggers will not be fired.
      */
 
-    // DROP SEQUENCE `graph_name_str`.`LABEL_ID_SEQ_NAME`
+    /* DROP SEQUENCE `graph_name_str`.`LABEL_ID_SEQ_NAME` */
     drop_stmt = makeNode(DropStmt);
     schema_name = makeString(get_graph_namespace_name(graph_name_str));
     label_id_seq_name = list_make2(schema_name, makeString(LABEL_ID_SEQ_NAME));
@@ -220,15 +255,15 @@ static void drop_schema_for_graph(char *graph_name_str, const bool cascade)
     drop_stmt->concurrent = false;
 
     RemoveRelations(drop_stmt);
-    // CommandCounterIncrement() is called in RemoveRelations()
+    /* CommandCounterIncrement() is called in RemoveRelations() */
 
-    // DROP SCHEMA `graph_name_str` [ CASCADE ]
+    /* DROP SCHEMA `graph_name_str` [ CASCADE ] */
     behavior = cascade ? DROP_CASCADE : DROP_RESTRICT;
     remove_schema((Node *)schema_name, behavior);
-    // CommandCounterIncrement() is called in performDeletion()
+    /* CommandCounterIncrement() is called in performDeletion() */
 }
 
-// See RemoveObjects() for more details.
+/* See RemoveObjects() for more details. */
 static void remove_schema(Node *schema_name, DropBehavior behavior)
 {
     ObjectAddress address;
@@ -236,12 +271,12 @@ static void remove_schema(Node *schema_name, DropBehavior behavior)
 
     address = get_object_address(OBJECT_SCHEMA, schema_name, &relation,
                                  AccessExclusiveLock, false);
-    // since the target object is always a schema, relation is NULL
+    /* since the target object is always a schema, relation is NULL */
     Assert(!relation);
 
     if (!OidIsValid(address.objectId))
     {
-        // missing_ok is always false
+        /* missing_ok is always false */
 
         /*
          * before calling this function, this condition is already checked in
@@ -253,7 +288,7 @@ static void remove_schema(Node *schema_name, DropBehavior behavior)
                                 strVal(schema_name))));
     }
 
-    // removeType is always OBJECT_SCHEMA
+    /* removeType is always OBJECT_SCHEMA */
 
     /*
      * Check permissions. Since the target object is always a schema, the
@@ -262,9 +297,9 @@ static void remove_schema(Node *schema_name, DropBehavior behavior)
     check_object_ownership(GetUserId(), OBJECT_SCHEMA, address, schema_name,
                            NULL);
 
-    // the target schema is not temporary
+    /* the target schema is not temporary */
 
-    // the target object is always a schema
+    /* the target object is always a schema */
 
     /*
      * set PERFORM_DELETION_INTERNAL flag so that object_access_hook can ignore
@@ -357,7 +392,7 @@ static void rename_graph(const Name graph_name, const Name new_name)
             (errmsg("graph \"%s\" renamed to \"%s\"", oldname, newname)));
 }
 
-// returns a list containing the name of every graph in the database
+/* returns a list containing the name of every graph in the database */
 List *get_graphnames(void)
 {
     TupleTableSlot *slot;
@@ -367,11 +402,11 @@ List *get_graphnames(void)
     List *graphnames = NIL;
     char *str;
 
-    ag_graph = heap_open(ag_graph_relation_id(), RowExclusiveLock);
+    ag_graph = table_open(ag_graph_relation_id(), RowExclusiveLock);
     scan_desc = systable_beginscan(ag_graph, ag_graph_name_index_id(), true,
                                    NULL, 0, NULL);
 
-    slot = MakeTupleTableSlot(RelationGetDescr(ag_graph));
+    slot = MakeTupleTableSlot(RelationGetDescr(ag_graph), &TTSOpsHeapTuple);
 
     for (;;)
     {
@@ -380,22 +415,22 @@ List *get_graphnames(void)
             break;
 
         ExecClearTuple(slot);
-        ExecStoreTuple(tuple, slot, InvalidBuffer, false);
+        ExecStoreHeapTuple(tuple, slot, false);
 
         slot_getallattrs(slot);
 
-        str = DatumGetCString(slot->tts_values[0]);
+        str = DatumGetCString(slot->tts_values[Anum_ag_graph_name - 1]);
         graphnames = lappend(graphnames, str);
     }
 
     ExecDropSingleTupleTableSlot(slot);
     systable_endscan(scan_desc);
-    heap_close(ag_graph, RowExclusiveLock);
+    table_close(ag_graph, RowExclusiveLock);
 
     return graphnames;
 }
 
-// deletes all the graphs in the list.
+/* deletes all the graphs in the list. */
 void drop_graphs(List *graphnames)
 {
     ListCell *lc;
