@@ -19,27 +19,15 @@
 
 #include "postgres.h"
 
-#include "access/heapam.h"
 #include "access/xact.h"
-#include "catalog/dependency.h"
 #include "catalog/namespace.h"
-#include "catalog/objectaddress.h"
 #include "catalog/pg_class_d.h"
 #include "commands/defrem.h"
 #include "commands/sequence.h"
 #include "commands/tablecmds.h"
 #include "miscadmin.h"
 #include "nodes/makefuncs.h"
-#include "nodes/nodes.h"
-#include "nodes/parsenodes.h"
-#include "nodes/pg_list.h"
-#include "nodes/plannodes.h"
-#include "nodes/primnodes.h"
-#include "nodes/value.h"
-#include "parser/parse_node.h"
 #include "parser/parser.h"
-#include "storage/lockdefs.h"
-#include "tcop/dest.h"
 #include "tcop/utility.h"
 #include "utils/acl.h"
 #include "utils/builtins.h"
@@ -50,8 +38,6 @@
 #include "catalog/ag_label.h"
 #include "commands/label_commands.h"
 #include "utils/ag_cache.h"
-#include "utils/agtype.h"
-#include "utils/graphid.h"
 #include "utils/name_validation.h"
 
 /*
@@ -65,7 +51,7 @@ static void create_table_for_label(char *graph_name, char *label_name,
                                    char *seq_name, char label_type,
                                    List *parents);
 
-// common
+/* common */
 static List *create_edge_table_elements(char *graph_name, char *label_name,
                                         char *schema_name, char *rel_name,
                                         char *seq_name);
@@ -87,14 +73,59 @@ static void change_label_id_default(char *graph_name, char *label_name,
                                     char *schema_name, char *seq_name,
                                     Oid relid);
 
-// drop
+/* drop */
 static void remove_relation(List *qname);
 static void range_var_callback_for_remove_relation(const RangeVar *rel,
                                                    Oid rel_oid,
                                                    Oid odl_rel_oid,
                                                    void *arg);
 
+PG_FUNCTION_INFO_V1(age_is_valid_label_name);
 
+Datum age_is_valid_label_name(PG_FUNCTION_ARGS)
+{
+    agtype *agt_arg = NULL;
+    agtype_value *agtv_value = NULL;
+    char *label_name = NULL;
+    bool is_valid = false;
+
+    if (PG_ARGISNULL(0))
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                errmsg("label name must not be NULL")));
+    }
+
+    agt_arg = AG_GET_ARG_AGTYPE_P(0);
+
+    if (!AGT_ROOT_IS_SCALAR(agt_arg))
+    {
+        ereport(ERROR,
+                (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                 errmsg("is_valid_label_name() only supports scalar arguments")));
+    }
+
+    agtv_value = get_ith_agtype_value_from_container(&agt_arg->root, 0);
+
+    if (agtv_value->type != AGTV_STRING)
+    {
+        ereport(ERROR,
+                (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                 errmsg("is_valid_label_name() only supports string arguments")));
+    }
+
+    label_name = pnstrdup(agtv_value->val.string.val,
+                          agtv_value->val.string.len);
+
+    is_valid = is_valid_label_name(label_name, 0);
+    pfree_if_not_null(label_name);
+
+    if (is_valid)
+    {
+        PG_RETURN_BOOL(true);
+    }
+
+    PG_RETURN_BOOL(false);
+}
 
 PG_FUNCTION_INFO_V1(create_vlabel);
 
@@ -110,68 +141,69 @@ PG_FUNCTION_INFO_V1(create_vlabel);
 
 Datum create_vlabel(PG_FUNCTION_ARGS)
 {
-    char *graph;
-    Name graph_name;
-    char *graph_name_str;
+    char *graph_name;
     Oid graph_oid;
     List *parent;
-
     RangeVar *rv;
+    char *label_name;
 
-    char *label;
-    Name label_name;
-    char *label_name_str;
-
-    // checking if user has not provided the graph name
+    /* checking if user has not provided the graph name */
     if (PG_ARGISNULL(0))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                 errmsg("graph name must not be NULL")));
     }
 
-    // checking if user has not provided the label name
+    /* checking if user has not provided the label name */
     if (PG_ARGISNULL(1))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                 errmsg("label name must not be NULL")));
     }
 
-    graph_name = PG_GETARG_NAME(0);
-    label_name = PG_GETARG_NAME(1);
+    graph_name = PG_GETARG_CSTRING(0);
+    label_name = PG_GETARG_CSTRING(1);
 
-    graph_name_str = NameStr(*graph_name);
-    label_name_str = NameStr(*label_name);
+    /* validate the graph and label names */
+    if (is_valid_graph_name(graph_name) == 0)
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                errmsg("graph name is invalid")));
+    }
 
-    // Check if graph does not exist
-    if (!graph_exists(graph_name_str))
+    if (is_valid_label_name(label_name, 0) == 0)
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                errmsg("label name is invalid")));
+    }
+
+    /* Check if graph does not exist */
+    if (!graph_exists(graph_name))
     {
         ereport(ERROR,
                 (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                        errmsg("graph \"%s\" does not exist.", graph_name_str)));
+                        errmsg("graph \"%s\" does not exist.", graph_name)));
     }
 
-    graph_oid = get_graph_oid(graph_name_str);
+    graph_oid = get_graph_oid(graph_name);
 
-    // Check if label with the input name already exists
-    if (label_exists(label_name_str, graph_oid))
+    /* Check if label with the input name already exists */
+    if (label_exists(label_name, graph_oid))
     {
         ereport(ERROR,
                 (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                        errmsg("label \"%s\" already exists", label_name_str)));
+                        errmsg("label \"%s\" already exists", label_name)));
     }
 
-    //Create the default label tables
-    graph = graph_name->data;
-    label = label_name->data;
-
-    rv = get_label_range_var(graph, graph_oid, AG_DEFAULT_LABEL_VERTEX);
+    /* Create the default label tables */
+    rv = get_label_range_var(graph_name, graph_oid, AG_DEFAULT_LABEL_VERTEX);
 
     parent = list_make1(rv);
 
-    create_label(graph, label, LABEL_TYPE_VERTEX, parent);
+    create_label(graph_name, label_name, LABEL_TYPE_VERTEX, parent);
 
     ereport(NOTICE,
-            (errmsg("VLabel \"%s\" has been created", NameStr(*label_name))));
+            (errmsg("VLabel \"%s\" has been created", label_name)));
 
     PG_RETURN_VOID();
 }
@@ -190,67 +222,68 @@ PG_FUNCTION_INFO_V1(create_elabel);
 
 Datum create_elabel(PG_FUNCTION_ARGS)
 {
-    char *graph;
-    Name graph_name;
-    char *graph_name_str;
+    char *graph_name;
     Oid graph_oid;
     List *parent;
-
     RangeVar *rv;
+    char *label_name;
 
-    char *label;
-    Name label_name;
-    char *label_name_str;
-
-    // checking if user has not provided the graph name
+    /* checking if user has not provided the graph name */
     if (PG_ARGISNULL(0))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                 errmsg("graph name must not be NULL")));
     }
 
-    // checking if user has not provided the label name
+    /* checking if user has not provided the label name */
     if (PG_ARGISNULL(1))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                 errmsg("label name must not be NULL")));
     }
 
-    graph_name = PG_GETARG_NAME(0);
-    label_name = PG_GETARG_NAME(1);
+    graph_name = PG_GETARG_CSTRING(0);
+    label_name = PG_GETARG_CSTRING(1);
 
-    graph_name_str = NameStr(*graph_name);
-    label_name_str = NameStr(*label_name);
+    /* validate the graph and label names */
+    if (is_valid_graph_name(graph_name) == 0)
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                errmsg("graph name is invalid")));
+    }
 
-    // Check if graph does not exist
-    if (!graph_exists(graph_name_str))
+    if (is_valid_label_name(label_name, 0) == 0)
+    {
+        ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                errmsg("label name is invalid")));
+    }
+
+    /* Check if graph does not exist */
+    if (!graph_exists(graph_name))
     {
         ereport(ERROR,
                 (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                 errmsg("graph \"%s\" does not exist.", graph_name_str)));
+                 errmsg("graph \"%s\" does not exist.", graph_name)));
     }
 
-    graph_oid = get_graph_oid(graph_name_str);
+    graph_oid = get_graph_oid(graph_name);
 
-    // Check if label with the input name already exists
-    if (label_exists(label_name_str, graph_oid))
+    /* Check if label with the input name already exists */
+    if (label_exists(label_name, graph_oid))
     {
         ereport(ERROR,
                 (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                        errmsg("label \"%s\" already exists", label_name_str)));
+                        errmsg("label \"%s\" already exists", label_name)));
     }
 
-    //Create the default label tables
-    graph = graph_name->data;
-    label = label_name->data;
-
-    rv = get_label_range_var(graph, graph_oid, AG_DEFAULT_LABEL_EDGE);
+    /* Create the default label tables */
+    rv = get_label_range_var(graph_name, graph_oid, AG_DEFAULT_LABEL_EDGE);
 
     parent = list_make1(rv);
-    create_label(graph, label, LABEL_TYPE_EDGE, parent);
+    create_label(graph_name, label_name, LABEL_TYPE_EDGE, parent);
 
     ereport(NOTICE,
-            (errmsg("ELabel \"%s\" has been created", NameStr(*label_name))));
+            (errmsg("ELabel \"%s\" has been created", label_name)));
 
     PG_RETURN_VOID();
 }
@@ -273,13 +306,7 @@ void create_label(char *graph_name, char *label_name, char label_type,
     int32 label_id;
     Oid relation_id;
 
-    if (!is_valid_label(label_name, label_type))
-    {
-        ereport(ERROR, (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                        errmsg("label name is invalid")));
-    }
-
-    if (!is_valid_label(label_name, label_type))
+    if (!is_valid_label_name(label_name, label_type))
     {
         ereport(ERROR, (errcode(ERRCODE_UNDEFINED_SCHEMA),
                         errmsg("label name is invalid")));
@@ -294,29 +321,29 @@ void create_label(char *graph_name, char *label_name, char label_type,
     graph_oid = cache_data->oid;
     nsp_id = cache_data->namespace;
 
-    // create a sequence for the new label to generate unique IDs for vertices
+    /* create a sequence for the new label to generate unique IDs for vertices */
     schema_name = get_namespace_name(nsp_id);
     rel_name = gen_label_relation_name(label_name);
     seq_name = ChooseRelationName(rel_name, "id", "seq", nsp_id, false);
     seq_range_var = makeRangeVar(schema_name, seq_name, -1);
     create_sequence_for_label(seq_range_var);
 
-    // create a table for the new label
+    /* create a table for the new label */
     create_table_for_label(graph_name, label_name, schema_name, rel_name,
                            seq_name, label_type, parents);
 
-    // record the new label in ag_label
+    /* record the new label in ag_label */
     relation_id = get_relname_relid(rel_name, nsp_id);
 
-    // If a label has parents, switch the parents id default, with its own.
+    /* If a label has parents, switch the parents id default, with its own. */
     if (list_length(parents) != 0)
         change_label_id_default(graph_name, label_name, schema_name, seq_name,
                                 relation_id);
 
-    // associate the sequence with the "id" column
+    /* associate the sequence with the "id" column */
     alter_sequence_owned_by_for_label(seq_range_var, rel_name);
 
-    // get a new "id" for the new label
+    /* get a new "id" for the new label */
     label_id = get_new_label_id(graph_oid, nsp_id);
 
     insert_label(label_name, graph_oid, label_id, label_type,
@@ -325,12 +352,14 @@ void create_label(char *graph_name, char *label_name, char label_type,
     CommandCounterIncrement();
 }
 
-// CREATE TABLE `schema_name`.`rel_name` (
-//   "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
-//   "start_id" graphid NOT NULL note: only for edge labels
-//   "end_id" graphid NOT NULL  note: only for edge labels
-//   "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
-// )
+/* 
+ * CREATE TABLE `schema_name`.`rel_name` (
+ * "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
+ * "start_id" graphid NOT NULL note: only for edge labels
+ * "end_id" graphid NOT NULL  note: only for edge labels
+ * "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
+ * )
+ */
 static void create_table_for_label(char *graph_name, char *label_name,
                                    char *schema_name, char *rel_name,
                                    char *seq_name, char label_type,
@@ -341,7 +370,7 @@ static void create_table_for_label(char *graph_name, char *label_name,
 
     create_stmt = makeNode(CreateStmt);
 
-    // relpersistence is set to RELPERSISTENCE_PERMANENT by makeRangeVar()
+    /* relpersistence is set to RELPERSISTENCE_PERMANENT by makeRangeVar() */
     create_stmt->relation = makeRangeVar(schema_name, rel_name, -1);
 
     /*
@@ -380,15 +409,17 @@ static void create_table_for_label(char *graph_name, char *label_name,
     ProcessUtility(wrapper, "(generated CREATE TABLE command)", false,
                    PROCESS_UTILITY_SUBCOMMAND, NULL, NULL, None_Receiver,
                    NULL);
-    // CommandCounterIncrement() is called in ProcessUtility()
+    /* CommandCounterIncrement() is called in ProcessUtility() */
 }
 
-// CREATE TABLE `schema_name`.`rel_name` (
-//   "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
-//   "start_id" graphid NOT NULL
-//   "end_id" graphid NOT NULL
-//   "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
-// )
+/* 
+ * CREATE TABLE `schema_name`.`rel_name` (
+ * "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
+ * "start_id" graphid NOT NULL
+ * "end_id" graphid NOT NULL
+ * "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
+ * )
+ */
 static List *create_edge_table_elements(char *graph_name, char *label_name,
                                         char *schema_name, char *rel_name,
                                         char *seq_name)
@@ -398,22 +429,22 @@ static List *create_edge_table_elements(char *graph_name, char *label_name,
     ColumnDef *end_id;
     ColumnDef *props;
 
-    // "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...)
+    /* "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...) */
     id = makeColumnDef(AG_EDGE_COLNAME_ID, GRAPHIDOID, -1, InvalidOid);
     id->constraints = list_make2(build_pk_constraint(),
                                  build_id_default(graph_name, label_name,
                                                   schema_name, seq_name));
 
-    // "start_id" graphid NOT NULL
+    /* "start_id" graphid NOT NULL */
     start_id = makeColumnDef(AG_EDGE_COLNAME_START_ID, GRAPHIDOID, -1,
                              InvalidOid);
     start_id->constraints = list_make1(build_not_null_constraint());
 
-    // "end_id" graphid NOT NULL
+    /* "end_id" graphid NOT NULL */
     end_id = makeColumnDef(AG_EDGE_COLNAME_END_ID, GRAPHIDOID, -1, InvalidOid);
     end_id->constraints = list_make1(build_not_null_constraint());
 
-    // "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
+    /* "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"() */
     props = makeColumnDef(AG_EDGE_COLNAME_PROPERTIES, AGTYPEOID, -1,
                           InvalidOid);
     props->constraints = list_make2(build_not_null_constraint(),
@@ -422,10 +453,12 @@ static List *create_edge_table_elements(char *graph_name, char *label_name,
     return list_make4(id, start_id, end_id, props);
 }
 
-// CREATE TABLE `schema_name`.`rel_name` (
-//   "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
-//   "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
-// )
+/* 
+ * CREATE TABLE `schema_name`.`rel_name` (
+ * "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...),
+ * "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
+ * )
+ */
 static List *create_vertex_table_elements(char *graph_name, char *label_name,
                                           char *schema_name, char *rel_name,
                                           char *seq_name)
@@ -433,13 +466,13 @@ static List *create_vertex_table_elements(char *graph_name, char *label_name,
     ColumnDef *id;
     ColumnDef *props;
 
-    // "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...)
+    /* "id" graphid PRIMARY KEY DEFAULT "ag_catalog"."_graphid"(...) */
     id = makeColumnDef(AG_VERTEX_COLNAME_ID, GRAPHIDOID, -1, InvalidOid);
     id->constraints = list_make2(build_pk_constraint(),
                                  build_id_default(graph_name, label_name,
                                                   schema_name, seq_name));
 
-    // "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"()
+    /* "properties" agtype NOT NULL DEFAULT "ag_catalog"."agtype_build_map"() */
     props = makeColumnDef(AG_VERTEX_COLNAME_PROPERTIES, AGTYPEOID, -1,
                           InvalidOid);
     props->constraints = list_make2(build_not_null_constraint(),
@@ -448,12 +481,13 @@ static List *create_vertex_table_elements(char *graph_name, char *label_name,
     return list_make2(id, props);
 }
 
-// CREATE SEQUENCE `seq_range_var` MAXVALUE `LOCAL_ID_MAX`
+/* CREATE SEQUENCE `seq_range_var` MAXVALUE `LOCAL_ID_MAX` */
 static void create_sequence_for_label(RangeVar *seq_range_var)
 {
     ParseState *pstate;
     CreateSeqStmt *seq_stmt;
-    char buf[32]; // greater than MAXINT8LEN+1
+    /* greater than MAXINT8LEN+1 */
+    char buf[32];
     DefElem *maxvalue;
 
     pstate = make_parsestate(NULL);
@@ -512,26 +546,26 @@ static FuncCall *build_id_default_func_expr(char *graph_name, char *label_name,
     List *graphid_func_args;
     FuncCall *graphid_func;
 
-    // Build a node that gets the label id
+    /* Build a node that gets the label id */
     label_id_func_name = list_make2(makeString("ag_catalog"),
                                     makeString("_label_id"));
     graph_name_const = makeNode(A_Const);
-    graph_name_const->val.type = T_String;
-    graph_name_const->val.val.str = graph_name;
+    graph_name_const->val.sval.type = T_String;
+    graph_name_const->val.sval.sval = graph_name;
     graph_name_const->location = -1;
     label_name_const = makeNode(A_Const);
-    label_name_const->val.type = T_String;
-    label_name_const->val.val.str = label_name;
+    label_name_const->val.sval.type = T_String;
+    label_name_const->val.sval.sval = label_name;
     label_name_const->location = -1;
     label_id_func_args = list_make2(graph_name_const, label_name_const);
     label_id_func = makeFuncCall(label_id_func_name, label_id_func_args, COERCE_SQL_SYNTAX, -1);
 
-    //Build a node that will get the next val from the label's sequence
+    /* Build a node that will get the next val from the label's sequence */
     nextval_func_name = SystemFuncName("nextval");
     qualified_seq_name = quote_qualified_identifier(schema_name, seq_name);
     qualified_seq_name_const = makeNode(A_Const);
-    qualified_seq_name_const->val.type = T_String;
-    qualified_seq_name_const->val.val.str = qualified_seq_name;
+    qualified_seq_name_const->val.sval.type = T_String;
+    qualified_seq_name_const->val.sval.sval = qualified_seq_name;
     qualified_seq_name_const->location = -1;
     regclass_cast = makeNode(TypeCast);
     regclass_cast->typeName = SystemTypeName("regclass");
@@ -573,7 +607,7 @@ static Constraint *build_id_default(char *graph_name, char *label_name,
     return id_default;
 }
 
-// NOT NULL
+/* NOT NULL */
 static Constraint *build_not_null_constraint(void)
 {
     Constraint *not_null;
@@ -585,14 +619,14 @@ static Constraint *build_not_null_constraint(void)
     return not_null;
 }
 
-// DEFAULT "ag_catalog"."agtype_build_map"()
+/* DEFAULT "ag_catalog"."agtype_build_map"() */
 static Constraint *build_properties_default(void)
 {
     List *func_name;
     FuncCall *func;
     Constraint *props_default;
 
-    // "ag_catalog"."agtype_build_map"()
+    /* "ag_catalog"."agtype_build_map"() */
     func_name = list_make2(makeString("ag_catalog"),
                            makeString("agtype_build_map"));
     func = makeFuncCall(func_name, NIL, COERCE_SQL_SYNTAX, -1);
@@ -649,7 +683,7 @@ static void change_label_id_default(char *graph_name, char *label_name,
     CommandCounterIncrement();
 }
 
-// CREATE SEQUENCE `seq_range_var` OWNED BY `schema_name`.`rel_name`."id"
+/* CREATE SEQUENCE `seq_range_var` OWNED BY `schema_name`.`rel_name`."id" */
 static void alter_sequence_owned_by_for_label(RangeVar *seq_range_var,
                                               char *rel_name)
 {
@@ -681,7 +715,7 @@ static int32 get_new_label_id(Oid graph_oid, Oid nsp_id)
     Oid seq_id;
     int cnt;
 
-    // get the OID of the sequence
+    /* get the OID of the sequence */
     seq_id = get_relname_relid(LABEL_ID_SEQ_NAME, nsp_id);
     if (!OidIsValid(seq_id))
     {
@@ -694,7 +728,7 @@ static int32 get_new_label_id(Oid graph_oid, Oid nsp_id)
     {
         int32 label_id;
 
-        // the data type of the sequence is integer (int4)
+        /* the data type of the sequence is integer (int4) */
         label_id = (int32) nextval_internal(seq_id, true);
         Assert(label_id_is_valid(label_id));
         if (!label_id_exists(graph_oid, label_id))
@@ -791,9 +825,9 @@ Datum drop_label(PG_FUNCTION_ARGS)
     qname = list_make2(makeString(schema_name), makeString(rel_name));
 
     remove_relation(qname);
-    // CommandCounterIncrement() is called in performDeletion()
+    /* CommandCounterIncrement() is called in performDeletion() */
 
-    // delete_label() will be called in object_access()
+    /* delete_label() will be called in object_access() */
 
     ereport(NOTICE, (errmsg("label \"%s\".\"%s\" has been dropped",
                             graph_name_str, label_name_str)));
@@ -801,18 +835,18 @@ Datum drop_label(PG_FUNCTION_ARGS)
     PG_RETURN_VOID();
 }
 
-// See RemoveRelations() for more details.
+/* See RemoveRelations() for more details. */
 static void remove_relation(List *qname)
 {
     RangeVar *rel;
     Oid rel_oid;
     ObjectAddress address;
 
-    AssertArg(list_length(qname) == 2);
+    Assert(list_length(qname) == 2);
 
-    // concurrent is false so lockmode is AccessExclusiveLock
+    /* concurrent is false so lockmode is AccessExclusiveLock */
 
-    // relkind is RELKIND_RELATION
+    /* relkind is RELKIND_RELATION */
 
     AcceptInvalidationMessages();
 
@@ -834,7 +868,7 @@ static void remove_relation(List *qname)
                                 rel->schemaname, rel->relname)));
     }
 
-    // concurrent is false
+    /* concurrent is false */
 
     ObjectAddressSet(address, RelationRelationId, rel_oid);
 
@@ -845,7 +879,7 @@ static void remove_relation(List *qname)
     performDeletion(&address, DROP_RESTRICT, PERFORM_DELETION_INTERNAL);
 }
 
-// See RangeVarCallbackForDropRelation() for more details.
+/* See RangeVarCallbackForDropRelation() for more details. */
 static void range_var_callback_for_remove_relation(const RangeVar *rel,
                                                    Oid rel_oid,
                                                    Oid odl_rel_oid,
@@ -857,28 +891,27 @@ static void range_var_callback_for_remove_relation(const RangeVar *rel,
      * always false. See RemoveRelations() for more details.
      */
 
-    // heapOid is always InvalidOid
+    /* heapOid is always InvalidOid */
 
-    // partParentOid is always InvalidOid
+    /* partParentOid is always InvalidOid */
 
     if (!OidIsValid(rel_oid))
         return;
 
-    // classform->relkind is always RELKIND_RELATION
+    /* classform->relkind is always RELKIND_RELATION */
 
-    // relkind == expected_relkind
+    /* relkind == expected_relkind */
 
-    if (!pg_class_ownercheck(rel_oid, GetUserId()) &&
-        !pg_namespace_ownercheck(get_rel_namespace(rel_oid), GetUserId()))
+    if (!object_ownercheck(rel_oid, get_rel_namespace(rel_oid), GetUserId()))
     {
         aclcheck_error(ACLCHECK_NOT_OWNER,
                        get_relkind_objtype(get_rel_relkind(rel_oid)),
                        rel->relname);
     }
 
-    // the target relation is not system class
+    /* the target relation is not system class */
 
-    // relkind is always RELKIND_RELATION
+    /* relkind is always RELKIND_RELATION */
 
-    // is_partition is false
+    /* is_partition is false */
 }
