@@ -19,30 +19,12 @@
 
 #include "postgres.h"
 
-#include "access/sysattr.h"
-#include "access/htup_details.h"
-#include "access/multixact.h"
-#include "access/xact.h"
 #include "storage/bufmgr.h"
-#include "executor/tuptable.h"
-#include "nodes/execnodes.h"
-#include "nodes/extensible.h"
-#include "nodes/nodes.h"
-#include "nodes/plannodes.h"
-#include "parser/parsetree.h"
-#include "parser/parse_relation.h"
-#include "rewrite/rewriteHandler.h"
-#include "utils/rel.h"
-#include "utils/tqual.h"
+#include "common/hashfn.h"
 
 #include "catalog/ag_label.h"
-#include "commands/label_commands.h"
 #include "executor/cypher_executor.h"
 #include "executor/cypher_utils.h"
-#include "parser/cypher_parse_node.h"
-#include "nodes/cypher_nodes.h"
-#include "utils/agtype.h"
-#include "utils/graphid.h"
 
 static void begin_cypher_delete(CustomScanState *node, EState *estate,
                                 int eflags);
@@ -89,16 +71,17 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
 
     Assert(list_length(css->cs->custom_plans) == 1);
 
-    // setup child
+    /* setup child */
     subplan = linitial(css->cs->custom_plans);
     node->ss.ps.lefttree = ExecInitNode(subplan, estate, eflags);
 
-    // setup expr context
+    /* setup expr context */
     ExecAssignExprContext(estate, &node->ss.ps);
 
-    // setup scan tuple slot and projection info
+    /* setup scan tuple slot and projection info */
     ExecInitScanTupleSlot(estate, &node->ss,
-                          ExecGetResultType(node->ss.ps.lefttree));
+                          ExecGetResultType(node->ss.ps.lefttree),
+                          &TTSOpsHeapTuple);
 
     if (!CYPHER_CLAUSE_IS_TERMINAL(css->flags))
     {
@@ -118,7 +101,7 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
     MemSet(&hashctl, 0, sizeof(hashctl));
     hashctl.keysize = sizeof(graphid);
     hashctl.entrysize =
-        sizeof(graphid); // entries are not used, but entrysize must >= keysize
+        sizeof(graphid); /* entries are not used, but entrysize must >= keysize */
     hashctl.hash = tag_hash;
     css->vertex_id_htab = hash_create(DELETE_VERTEX_HTAB_NAME,
                                       DELETE_VERTEX_HTAB_SIZE, &hashctl,
@@ -160,7 +143,7 @@ static TupleTableSlot *exec_cypher_delete(CustomScanState *node)
          */
         while(true)
         {
-            //Process the subtree first
+            /* Process the subtree first */
             Decrement_Estate_CommandId(estate)
             slot = ExecProcNode(node->ss.ps.lefttree);
             Increment_Estate_CommandId(estate)
@@ -168,7 +151,7 @@ static TupleTableSlot *exec_cypher_delete(CustomScanState *node)
             if (TupIsNull(slot))
                 break;
 
-            // setup the scantuple that the process_delete_list needs
+            /* setup the scantuple that the process_delete_list needs */
             econtext->ecxt_scantuple =
                 node->ss.ps.lefttree->ps_ProjInfo->pi_exprContext->ecxt_scantuple;
 
@@ -179,7 +162,7 @@ static TupleTableSlot *exec_cypher_delete(CustomScanState *node)
     }
     else
     {
-        //Process the subtree first
+        /* Process the subtree first */
         Decrement_Estate_CommandId(estate)
         slot = ExecProcNode(node->ss.ps.lefttree);
         Increment_Estate_CommandId(estate)
@@ -187,7 +170,7 @@ static TupleTableSlot *exec_cypher_delete(CustomScanState *node)
         if (TupIsNull(slot))
             return NULL;
 
-        // setup the scantuple that the process_delete_list needs
+        /* setup the scantuple that the process_delete_list needs */
         econtext->ecxt_scantuple =
             node->ss.ps.lefttree->ps_ProjInfo->pi_exprContext->ecxt_scantuple;
 
@@ -243,7 +226,7 @@ Node *create_cypher_delete_plan_state(CustomScan *cscan)
 
     cypher_css->cs = cscan;
 
-    // get the serialized data structure from the Const and deserialize it.
+    /* get the serialized data structure from the Const and deserialize it. */
     c = linitial(cscan->custom_private);
     serialized_data = (char *)c->constvalue;
     delete_data = stringToNode(serialized_data);
@@ -273,7 +256,7 @@ static agtype_value *extract_entity(CustomScanState *node,
 
     tupleDescriptor = scanTupleSlot->tts_tupleDescriptor;
 
-    // type checking, make sure the entity is an agtype vertex or edge
+    /* type checking, make sure the entity is an agtype vertex or edge */
     if (tupleDescriptor->attrs[entity_position -1].atttypid != AGTYPEOID)
         ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
                 errmsg("DELETE clause can only delete agtype")));
@@ -295,16 +278,16 @@ static agtype_value *extract_entity(CustomScanState *node,
 static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
                           HeapTuple tuple)
 {
-    ResultRelInfo *saved_resultRelInfo;
+    ResultRelInfo **saved_resultRels;
     LockTupleMode lockmode;
-    HeapUpdateFailureData hufd;
-    HTSU_Result lock_result;
-    HTSU_Result delete_result;
+    TM_FailureData hufd;
+    TM_Result lock_result;
+    TM_Result delete_result;
     Buffer buffer;
 
-    // Find the physical tuple, this variable is coming from
-    saved_resultRelInfo = estate->es_result_relation_info;
-    estate->es_result_relation_info = resultRelInfo;
+    /* Find the physical tuple, this variable is coming from */
+    saved_resultRels = estate->es_result_relations;
+    estate->es_result_relations = &resultRelInfo;
 
     lockmode = ExecUpdateLockMode(estate, resultRelInfo);
 
@@ -316,11 +299,11 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
      * It is possible the entity may have already been deleted. If the tuple
      * can be deleted, the lock result will be HeapTupleMayBeUpdated. If the
      * tuple was already deleted by this DELETE clause, the result would be
-     * HeapTupleSelfUpdated, if the result was deleted by a previous delete
-     * clause, the result will HeapTupleInvisible. Throw an error if any
+     * TM_SelfModified, if the result was deleted by a previous delete
+     * clause, the result will TM_Invisible. Throw an error if any
      * other result was returned.
      */
-    if (lock_result == HeapTupleMayBeUpdated)
+    if (lock_result == TM_Ok)
     {
         delete_result = heap_delete(resultRelInfo->ri_RelationDesc,
                                     &tuple->t_self, GetCurrentCommandId(true),
@@ -333,30 +316,32 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
          */
         switch (delete_result)
         {
-                case HeapTupleMayBeUpdated:
-                        break;
-                case HeapTupleSelfUpdated:
-                        ereport(ERROR,
-                                (errcode(ERRCODE_INTERNAL_ERROR),
-                                         errmsg("deleting the same entity more than once cannot happen")));
-                        /* ereport never gets here */
-                        break;
-                case HeapTupleUpdated:
-                        ereport(ERROR,
-                                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
-                                         errmsg("could not serialize access due to concurrent update")));
-                        /* ereport never gets here */
-                        break;
-                default:
-                        elog(ERROR, "Entity failed to be update");
-                        /* elog never gets here */
-                        break;
+        case TM_Ok:
+            break;
+        case TM_SelfModified:
+            ereport(
+                ERROR,
+                (errcode(ERRCODE_INTERNAL_ERROR),
+                 errmsg(
+                     "deleting the same entity more than once cannot happen")));
+            /* ereport never gets here */
+            break;
+        case TM_Updated:
+            ereport(
+                ERROR,
+                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
+                 errmsg("could not serialize access due to concurrent update")));
+            /* ereport never gets here */
+            break;
+        default:
+            elog(ERROR, "Entity failed to be update");
+            /* elog never gets here */
+            break;
         }
         /* increment the command counter */
         CommandCounterIncrement();
     }
-    else if (lock_result != HeapTupleInvisible &&
-             lock_result != HeapTupleSelfUpdated)
+    else if (lock_result != TM_Invisible && lock_result != TM_SelfModified)
     {
         ereport(ERROR,
                 (errcode(ERRCODE_INTERNAL_ERROR),
@@ -366,7 +351,7 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
 
     ReleaseBuffer(buffer);
 
-    estate->es_result_relation_info = saved_resultRelInfo;
+    estate->es_result_relations = saved_resultRels;
 }
 
 /*
@@ -387,7 +372,7 @@ static void process_delete_list(CustomScanState *node)
         cypher_delete_item *item;
         agtype_value *original_entity_value, *id, *label;
         ScanKeyData scan_keys[1];
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         ResultRelInfo *resultRelInfo;
         HeapTuple heap_tuple;
         char *label_name;
@@ -439,8 +424,8 @@ static void process_delete_list(CustomScanState *node)
          */
         estate->es_snapshot->curcid = GetCurrentCommandId(false);
         estate->es_output_cid = GetCurrentCommandId(false);
-        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
-                                   estate->es_snapshot, 1, scan_keys);
+        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
+                                    estate->es_snapshot, 1, scan_keys);
 
         /* Retrieve the tuple. */
         heap_tuple = heap_getnext(scan_desc, ForwardScanDirection);
@@ -452,7 +437,7 @@ static void process_delete_list(CustomScanState *node)
          */
         if (!HeapTupleIsValid(heap_tuple))
         {
-            heap_endscan(scan_desc);
+            table_endscan(scan_desc);
             destroy_entity_result_rel_info(resultRelInfo);
 
             continue;
@@ -474,7 +459,7 @@ static void process_delete_list(CustomScanState *node)
         delete_entity(estate, resultRelInfo, heap_tuple);
 
         /* Close the scan and the relation. */
-        heap_endscan(scan_desc);
+        table_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
 }
@@ -497,21 +482,19 @@ static void check_for_connected_edges(CustomScanState *node)
     {
         char *label_name = lfirst(lc);
         ResultRelInfo *resultRelInfo;
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         HeapTuple tuple;
         TupleTableSlot *slot;
 
-        resultRelInfo = create_entity_result_rel_info(estate,
-                                                      graph_name, label_name);
-
+        resultRelInfo = create_entity_result_rel_info(estate, graph_name,
+                                                      label_name);
         estate->es_snapshot->curcid = GetCurrentCommandId(false);
         estate->es_output_cid = GetCurrentCommandId(false);
-
-        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
-                                   estate->es_snapshot, 0, NULL);
-
-        slot = ExecInitExtraTupleSlot(estate,
-                    RelationGetDescr(resultRelInfo->ri_RelationDesc));
+        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
+                                    estate->es_snapshot, 0, NULL);
+        slot = ExecInitExtraTupleSlot(
+            estate, RelationGetDescr(resultRelInfo->ri_RelationDesc),
+            &TTSOpsHeapTuple);
 
         /* for each row */
         while (true)
@@ -530,7 +513,7 @@ static void check_for_connected_edges(CustomScanState *node)
                 break;
             }
 
-            ExecStoreTuple(tuple, slot, InvalidBuffer, false);
+            ExecStoreHeapTuple(tuple, slot, false);
 
             startid = GRAPHID_GET_DATUM(slot_getattr(
                 slot, Anum_ag_label_edge_table_start_id, &isNull));
@@ -564,7 +547,7 @@ static void check_for_connected_edges(CustomScanState *node)
             }
         }
 
-        heap_endscan(scan_desc);
+        table_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
 }
