@@ -19,10 +19,6 @@
 
 #include "postgres.h"
 
-#include "lib/stringinfo.h"
-#include "nodes/extensible.h"
-#include "nodes/nodes.h"
-
 #include "nodes/cypher_nodes.h"
 #include "nodes/cypher_outfuncs.h"
 
@@ -45,7 +41,7 @@ static void outChar(StringInfo str, char c);
         outToken(str, _node->field_name); \
     } while (0)
 
-// Write a char field (ie, one ascii character)
+/* Write a char field (ie, one ascii character) */
 #define WRITE_CHAR_FIELD(fldname) \
     do { \
         (appendStringInfo(str, " :" CppAsString(fldname) " "), \
@@ -59,7 +55,7 @@ static void outChar(StringInfo str, char c);
                          _node->field_name ? "true" : "false"); \
     } while (0)
 
-// write an enumerated-type field as an integer code
+/* write an enumerated-type field as an integer code */
 #define WRITE_ENUM_FIELD(field_name, enum_type) \
     do \
     { \
@@ -81,14 +77,14 @@ static void outChar(StringInfo str, char c);
                          _node->field_name); \
     } while (0)
 
-// Write an integer field (anything written as ":fldname %d")
+/* Write an integer field (anything written as ":fldname %d") */
 #define WRITE_INT32_FIELD(field_name) \
     do { \
         appendStringInfo(str, " :" CppAsString(field_name) " %d", \
                          _node->field_name); \
     } while (0)
 
-// Write an OID field (don't hard-wire assumption that OID is same as uint)
+/* Write an OID field (don't hard-wire assumption that OID is same as uint) */
 #define WRITE_OID_FIELD(fldname) \
     do { \
         appendStringInfo(str, " :" CppAsString(fldname) " %u", \
@@ -97,7 +93,7 @@ static void outChar(StringInfo str, char c);
 
 
 
-// serialization function for the cypher_return ExtensibleNode.
+/* serialization function for the cypher_return ExtensibleNode. */
 void out_cypher_return(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_return);
@@ -114,7 +110,7 @@ void out_cypher_return(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(rarg);
 }
 
-// serialization function for the cypher_with ExtensibleNode.
+/* serialization function for the cypher_with ExtensibleNode. */
 void out_cypher_with(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_with);
@@ -127,7 +123,7 @@ void out_cypher_with(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(where);
 }
 
-// serialization function for the cypher_match ExtensibleNode.
+/* serialization function for the cypher_match ExtensibleNode. */
 void out_cypher_match(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_match);
@@ -137,7 +133,7 @@ void out_cypher_match(StringInfo str, const ExtensibleNode *node)
     WRITE_BOOL_FIELD(optional);
 }
 
-// serialization function for the cypher_create ExtensibleNode.
+/* serialization function for the cypher_create ExtensibleNode. */
 void out_cypher_create(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_create);
@@ -145,7 +141,7 @@ void out_cypher_create(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(pattern);
 }
 
-// serialization function for the cypher_set ExtensibleNode.
+/* serialization function for the cypher_set ExtensibleNode. */
 void out_cypher_set(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_set);
@@ -154,7 +150,7 @@ void out_cypher_set(StringInfo str, const ExtensibleNode *node)
     WRITE_BOOL_FIELD(is_remove);
 }
 
-// serialization function for the cypher_set_item ExtensibleNode.
+/* serialization function for the cypher_set_item ExtensibleNode. */
 void out_cypher_set_item(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_set_item);
@@ -164,7 +160,7 @@ void out_cypher_set_item(StringInfo str, const ExtensibleNode *node)
     WRITE_BOOL_FIELD(is_add);
 }
 
-// serialization function for the cypher_delete ExtensibleNode.
+/* serialization function for the cypher_delete ExtensibleNode. */
 void out_cypher_delete(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_delete);
@@ -180,7 +176,19 @@ void out_cypher_unwind(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(target);
 }
 
-// serialization function for the cypher_delete ExtensibleNode.
+/* serialization function for the cypher_list_comprehension ExtensibleNode. */
+void out_cypher_list_comprehension(StringInfo str, const ExtensibleNode *node)
+{
+    DEFINE_AG_NODE(cypher_list_comprehension);
+
+    WRITE_STRING_FIELD(varname);
+    WRITE_NODE_FIELD(expr);
+    WRITE_NODE_FIELD(where);
+    WRITE_NODE_FIELD(mapping_expr);
+}
+
+
+/* serialization function for the cypher_delete ExtensibleNode. */
 void out_cypher_merge(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_merge);
@@ -188,7 +196,7 @@ void out_cypher_merge(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(path);
 }
 
-// serialization function for the cypher_path ExtensibleNode.
+/* serialization function for the cypher_path ExtensibleNode. */
 void out_cypher_path(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_path);
@@ -199,7 +207,7 @@ void out_cypher_path(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_node ExtensibleNode.
+/* serialization function for the cypher_node ExtensibleNode. */
 void out_cypher_node(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_node);
@@ -212,7 +220,7 @@ void out_cypher_node(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_relationship ExtensibleNode.
+/* serialization function for the cypher_relationship ExtensibleNode. */
 void out_cypher_relationship(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_relationship);
@@ -227,7 +235,7 @@ void out_cypher_relationship(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_bool_const ExtensibleNode.
+/* serialization function for the cypher_bool_const ExtensibleNode. */
 void out_cypher_bool_const(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_bool_const);
@@ -236,7 +244,7 @@ void out_cypher_bool_const(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_param ExtensibleNode.
+/* serialization function for the cypher_param ExtensibleNode. */
 void out_cypher_param(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_param);
@@ -245,7 +253,7 @@ void out_cypher_param(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_map ExtensibleNode.
+/* serialization function for the cypher_map ExtensibleNode. */
 void out_cypher_map(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_map);
@@ -254,7 +262,16 @@ void out_cypher_map(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_list ExtensibleNode.
+void out_cypher_map_projection(StringInfo str, const ExtensibleNode *node)
+{
+    DEFINE_AG_NODE(cypher_map_projection);
+
+    WRITE_NODE_FIELD(map_var);
+    WRITE_NODE_FIELD(map_elements);
+    WRITE_LOCATION_FIELD(location);
+}
+
+/* serialization function for the cypher_list ExtensibleNode. */
 void out_cypher_list(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_list);
@@ -263,7 +280,7 @@ void out_cypher_list(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_comparison_aexpr ExtensibleNode.
+/* serialization function for the cypher_comparison_aexpr ExtensibleNode. */
 void out_cypher_comparison_aexpr(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_comparison_aexpr);
@@ -275,7 +292,7 @@ void out_cypher_comparison_aexpr(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_comparison_boolexpr ExtensibleNode.
+/* serialization function for the cypher_comparison_boolexpr ExtensibleNode. */
 void out_cypher_comparison_boolexpr(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_comparison_boolexpr);
@@ -285,7 +302,7 @@ void out_cypher_comparison_boolexpr(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_string_match ExtensibleNode.
+/* serialization function for the cypher_string_match ExtensibleNode. */
 void out_cypher_string_match(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_string_match);
@@ -296,17 +313,17 @@ void out_cypher_string_match(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_typecast ExtensibleNode.
+/* serialization function for the cypher_typecast ExtensibleNode. */
 void out_cypher_typecast(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_typecast);
 
     WRITE_NODE_FIELD(expr);
-    WRITE_STRING_FIELD(typecast);
+    WRITE_NODE_FIELD(typname);
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_integer_const ExtensibleNode.
+/* serialization function for the cypher_integer_const ExtensibleNode. */
 void out_cypher_integer_const(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_integer_const);
@@ -315,7 +332,7 @@ void out_cypher_integer_const(StringInfo str, const ExtensibleNode *node)
     WRITE_LOCATION_FIELD(location);
 }
 
-// serialization function for the cypher_sub_pattern ExtensibleNode.
+/* serialization function for the cypher_sub_pattern ExtensibleNode. */
 void out_cypher_sub_pattern(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_sub_pattern);
@@ -324,7 +341,16 @@ void out_cypher_sub_pattern(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(pattern);
 }
 
-// serialization function for the cypher_call ExtensibleNode.
+/* serialization function for the cypher_sub_pattern ExtensibleNode. */
+void out_cypher_sub_query(StringInfo str, const ExtensibleNode *node)
+{
+    DEFINE_AG_NODE(cypher_sub_query);
+
+    WRITE_ENUM_FIELD(kind, csp_kind);
+    WRITE_NODE_FIELD(query);
+}
+
+/* serialization function for the cypher_call ExtensibleNode. */
 void out_cypher_call(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_call);
@@ -335,17 +361,17 @@ void out_cypher_call(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(yield_items);
 }
 
-// serialization function for the cypher_create_target_nodes ExtensibleNode.
+/* serialization function for the cypher_create_target_nodes ExtensibleNode. */
 void out_cypher_create_target_nodes(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_create_target_nodes);
 
     WRITE_NODE_FIELD(paths);
     WRITE_INT32_FIELD(flags);
-    WRITE_OID_FIELD(graph_oid);
+    WRITE_INT32_FIELD(graph_oid);
 }
 
-// serialization function for the cypher_create_path ExtensibleNode.
+/* serialization function for the cypher_create_path ExtensibleNode. */
 void out_cypher_create_path(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_create_path);
@@ -355,7 +381,7 @@ void out_cypher_create_path(StringInfo str, const ExtensibleNode *node)
     WRITE_STRING_FIELD(var_name);
 }
 
-// serialization function for the cypher_target_node ExtensibleNode.
+/* serialization function for the cypher_target_node ExtensibleNode. */
 void out_cypher_target_node(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_target_node);
@@ -376,7 +402,7 @@ void out_cypher_target_node(StringInfo str, const ExtensibleNode *node)
     WRITE_INT32_FIELD(tuple_position);
 }
 
-// serialization function for the cypher_update_information ExtensibleNode.
+/* serialization function for the cypher_update_information ExtensibleNode. */
 void out_cypher_update_information(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_update_information);
@@ -388,7 +414,7 @@ void out_cypher_update_information(StringInfo str, const ExtensibleNode *node)
     WRITE_STRING_FIELD(clause_name);
 }
 
-// serialization function for the cypher_update_item ExtensibleNode.
+/* serialization function for the cypher_update_item ExtensibleNode. */
 void out_cypher_update_item(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_update_item);
@@ -402,7 +428,7 @@ void out_cypher_update_item(StringInfo str, const ExtensibleNode *node)
     WRITE_BOOL_FIELD(is_add);
 }
 
-// serialization function for the cypher_delete_information ExtensibleNode.
+/* serialization function for the cypher_delete_information ExtensibleNode. */
 void out_cypher_delete_information(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_delete_information);
@@ -410,11 +436,11 @@ void out_cypher_delete_information(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(delete_items);
     WRITE_INT32_FIELD(flags);
     WRITE_STRING_FIELD(graph_name);
-    WRITE_OID_FIELD(graph_oid);
+    WRITE_INT32_FIELD(graph_oid);
     WRITE_BOOL_FIELD(detach);
 }
 
-// serialization function for the cypher_delete_item ExtensibleNode.
+/* serialization function for the cypher_delete_item ExtensibleNode. */
 void out_cypher_delete_item(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_delete_item);
@@ -423,13 +449,13 @@ void out_cypher_delete_item(StringInfo str, const ExtensibleNode *node)
     WRITE_STRING_FIELD(var_name);
 }
 
-// serialization function for the cypher_merge_information ExtensibleNode.
+/* serialization function for the cypher_merge_information ExtensibleNode. */
 void out_cypher_merge_information(StringInfo str, const ExtensibleNode *node)
 {
     DEFINE_AG_NODE(cypher_merge_information);
 
     WRITE_INT32_FIELD(flags);
-    WRITE_OID_FIELD(graph_oid);
+    WRITE_INT32_FIELD(graph_oid);
     WRITE_INT32_FIELD(merge_function_attr);
     WRITE_NODE_FIELD(path);
 }
