@@ -35,6 +35,8 @@
 #include "utils/ag_func.h"
 #include "utils/age_session_info.h"
 
+typedef bool (*cypher_expression_condition)(Node *expr);
+
 /*
  * extra_node is a global variable to this source to store, at the moment, the
  * explain stmt node passed up by the parser. The return value from the parser
@@ -49,7 +51,7 @@ static void build_explain_query(Query *query, Node *explain_node);
 
 static post_parse_analyze_hook_type prev_post_parse_analyze_hook;
 
-static void post_parse_analyze(ParseState *pstate, Query *query);
+static void post_parse_analyze(ParseState *pstate, Query *query, JumbleState *jstate);
 static bool convert_cypher_walker(Node *node, ParseState *pstate);
 static bool is_rte_cypher(RangeTblEntry *rte);
 static bool is_func_cypher(FuncExpr *funcexpr);
@@ -63,10 +65,9 @@ static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
 static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
                                         ParseState *parent_pstate,
                                         const char *query_str, int query_loc,
-                                        char *graph_name, Oid graph_oid,
+                                        char *graph_name, uint32 graph_oid,
                                         Param *params);
 
-
 void post_parse_analyze_init(void)
 {
     prev_post_parse_analyze_hook = post_parse_analyze_hook;
@@ -78,11 +79,11 @@ void post_parse_analyze_fini(void)
     post_parse_analyze_hook = prev_post_parse_analyze_hook;
 }
 
-static void post_parse_analyze(ParseState *pstate, Query *query)
+static void post_parse_analyze(ParseState *pstate, Query *query, JumbleState *jstate)
 {
     if (prev_post_parse_analyze_hook)
     {
-        prev_post_parse_analyze_hook(pstate, query);
+        prev_post_parse_analyze_hook(pstate, query, jstate);
     }
 
     /*
@@ -106,12 +107,12 @@ static void post_parse_analyze(ParseState *pstate, Query *query)
         }
 
         /* reset extra_node */
-        pfree(extra_node);
+        pfree_if_not_null(extra_node);
         extra_node = NULL;
     }
 }
 
-// find cypher() calls in FROM clauses and convert them to SELECT subqueries
+/* find cypher() calls in FROM clauses and convert them to SELECT subqueries */
 static bool convert_cypher_walker(Node *node, ParseState *pstate)
 {
     if (!node)
@@ -124,7 +125,7 @@ static bool convert_cypher_walker(Node *node, ParseState *pstate)
         switch (rte->rtekind)
         {
         case RTE_SUBQUERY:
-            // traverse other RTE_SUBQUERYs
+            /* traverse other RTE_SUBQUERYs */
             return convert_cypher_walker((Node *)rte->subquery, pstate);
         case RTE_FUNCTION:
             if (is_rte_cypher(rte))
@@ -211,6 +212,27 @@ static bool convert_cypher_walker(Node *node, ParseState *pstate)
         bool result = false;
         Query *query = (Query *)node;
 
+        /*
+         * If this is a utility command, we need to unwrap the internal query
+         * and pass it as the query.
+         *
+         * NOTE: This code only "knows" about the following utility commands -
+         *
+         * CREATE TABLE AS
+         *
+         * Others need to be added on a case by case basis.
+         */
+        if (query->utilityStmt != NULL &&
+            IsA(query->utilityStmt, CreateTableAsStmt))
+        {
+            CreateTableAsStmt *ctas = (CreateTableAsStmt *)query->utilityStmt;
+
+            if (IsA(ctas->query, Query))
+            {
+                query = (Query *)ctas->query;
+            }
+        }
+
         /*
          * QTW_EXAMINE_RTES
          *     We convert RTE_FUNCTION (cypher()) to RTE_SUBQUERY (SELECT)
@@ -275,7 +297,7 @@ static void build_explain_query(Query *query, Node *explain_node)
     ((ExplainStmt *)explain_node)->options = NULL;
 
     /* we need to free query_node as it is no longer needed */
-    pfree(query_node);
+    pfree_if_not_null(query_node);
 }
 
 static bool is_rte_cypher(RangeTblEntry *rte)
@@ -333,7 +355,7 @@ static bool is_func_cypher(FuncExpr *funcexpr)
     return is_oid_ag_func(funcexpr->funcid, "cypher");
 }
 
-// convert cypher() call to SELECT subquery in-place
+/* convert cypher() call to SELECT subquery in-place */
 static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
 {
     RangeTblFunction *rtfunc = linitial(rte->functions);
@@ -546,7 +568,7 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     if (extra_node == NULL)
     {
         extra_node = llast(stmt);
-        list_delete_ptr(stmt, extra_node);
+        stmt = list_delete_ptr(stmt, extra_node);
     }
     else
     {
@@ -556,15 +578,15 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
                 (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                  errmsg("too many extra_nodes passed from parser")));
 
-        list_delete_ptr(stmt, temp);
+        stmt = list_delete_ptr(stmt, temp);
     }
 
     cancel_errpos_ecb(&ecb_state);
 
     Assert(pstate->p_expr_kind == EXPR_KIND_NONE);
     pstate->p_expr_kind = EXPR_KIND_FROM_SUBSELECT;
-    // transformRangeFunction() always sets p_lateral_active to true.
-    // FYI, rte is RTE_FUNCTION and is being converted to RTE_SUBQUERY here.
+    /* transformRangeFunction() always sets p_lateral_active to true. */
+    /* FYI, rte is RTE_FUNCTION and is being converted to RTE_SUBQUERY here. */
     pstate->p_lateral_active = true;
 
     /*
@@ -575,7 +597,7 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     if (is_ag_node(llast(stmt), cypher_create) || is_ag_node(llast(stmt), cypher_set) ||
         is_ag_node(llast(stmt), cypher_delete) || is_ag_node(llast(stmt), cypher_merge))
     {
-        // column definition list must be ... AS relname(colname agtype) ...
+        /* column definition list must be ... AS relname(colname agtype) ... */
         if (!(rtfunc->funccolcount == 1 &&
               linitial_oid(rtfunc->funccoltypes) == AGTYPEOID))
         {
@@ -599,10 +621,10 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     pstate->p_lateral_active = false;
     pstate->p_expr_kind = EXPR_KIND_NONE;
 
-    // rte->functions and rte->funcordinality are kept for debugging.
-    // rte->alias, rte->eref, and rte->lateral need to be the same.
-    // rte->inh is always false for both RTE_FUNCTION and RTE_SUBQUERY.
-    // rte->inFromCl is always true for RTE_FUNCTION.
+    /* rte->functions and rte->funcordinality are kept for debugging. */
+    /* rte->alias, rte->eref, and rte->lateral need to be the same. */
+    /* rte->inh is always false for both RTE_FUNCTION and RTE_SUBQUERY. */
+    /* rte->inFromCl is always true for RTE_FUNCTION. */
     rte->rtekind = RTE_SUBQUERY;
     rte->subquery = query;
 }
@@ -653,6 +675,221 @@ static int get_query_location(const int location, const char *source_str)
     return strchr(p + 1, '$') - source_str + 1;
 }
 
+
+/*
+ * This is an extension of postgres's raw_expression_tree_walker() function.
+ * It can walk cypher-specific nodes found in the expression tree during
+ * parse analysis.
+ * 
+ * More nodes can be added to this function as needed.
+ */
+bool cypher_raw_expr_tree_walker(Node *node,
+                                 bool (*walker)(),
+                                 void *context)
+{
+    ListCell *temp;
+
+    if (node == NULL)
+        return false;
+
+#define WALK(n) walker((Node *) (n), context)
+#define LIST_WALK(l) cypher_raw_expr_tree_walker((Node *) (l), walker, context)
+
+    if (IsA(node, ExtensibleNode))
+    {
+        if (is_ag_node(node, cypher_bool_const) ||
+            is_ag_node(node, cypher_integer_const) ||
+            is_ag_node(node, cypher_param) ||
+            is_ag_node(node, cypher_sub_pattern) ||
+            is_ag_node(node, cypher_sub_query))
+            /* Add more non-recursible node types here as needed */
+        {
+            return false;
+        }
+        else if (is_ag_node(node, cypher_map))
+        {
+            cypher_map *cm = (cypher_map *)node;
+            ListCell *le;
+
+            Assert(list_length(cm->keyvals) % 2 == 0);
+
+            le = list_head(cm->keyvals);
+
+            while(le != NULL)
+            {
+                Node *val;
+
+                le = lnext(cm->keyvals, le);
+
+                val = lfirst(le);
+
+                if (WALK(val))
+                {
+                    return true;
+                }
+
+                le = lnext(cm->keyvals, le);
+
+            }
+        }
+        else if (is_ag_node(node, cypher_map_projection))
+        {
+            cypher_map_projection *cmp = (cypher_map_projection *)node;
+
+            if (LIST_WALK(cmp->map_elements))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_list))
+        {
+            cypher_list *cl = (cypher_list *)node;
+            
+            if (LIST_WALK(cl->elems))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_string_match))
+        {
+            cypher_string_match *csm = (cypher_string_match *)node;
+
+            if (WALK(csm->lhs))
+            {
+                return true;
+            }
+
+            if (WALK(csm->rhs))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_typecast))
+        {
+            cypher_typecast *t = (cypher_typecast *)node;
+
+            if (WALK(t->expr))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_comparison_aexpr))
+        {
+            cypher_comparison_aexpr *a = (cypher_comparison_aexpr *)node;
+
+            if (WALK(a->lexpr))
+            {
+                return true;
+            }
+
+            if (WALK(a->rexpr))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_comparison_boolexpr))
+        {
+            cypher_comparison_boolexpr *b = (cypher_comparison_boolexpr *)node;
+            
+            if (LIST_WALK(b->args))
+            {
+                return true;
+            }
+        }
+        else if (is_ag_node(node, cypher_unwind))
+        {
+            cypher_unwind *unw = (cypher_unwind *)node;
+
+            if (WALK(unw->target))
+            {
+                return true;
+            }
+        }
+
+        else if (is_ag_node(node, cypher_list_comprehension))
+        {
+            cypher_list_comprehension *lc = (cypher_list_comprehension *)node;
+
+            if (WALK(lc->expr))
+            {
+                return true;
+            }
+
+            if (WALK(lc->where))
+            {
+                return true;
+            }
+
+            if (WALK(lc->mapping_expr))
+            {
+                return true;
+            }
+        }
+        /* Add more node types here as needed */
+        else
+        {
+            ereport(ERROR,
+                (errmsg_internal("unrecognized ExtensibleNode: %s",
+                                 ((ExtensibleNode *)node)->extnodename)));
+        }
+    }
+    /*
+     * postgres's raw expresssion tree walker does not handle List
+     */
+    else if (IsA(node, List))
+    {
+        foreach(temp, (List *) node)
+        {
+            if (WALK((Node *) lfirst(temp)))
+                return true;
+        }
+    }
+
+#undef LIST_WALK
+    else
+    {
+        return raw_expression_tree_walker(node, walker, context);
+    }
+    
+    return false;
+}
+
+/*
+ * This is an extension of postgres's expression_tree_walker() function.
+ * It is meant to walk cypher-specific nodes found in the expression tree
+ * post parse analysis.
+ *
+ * More nodes can be added to this function as needed.
+ */
+bool cypher_expr_tree_walker(Node *node,
+                             bool (*walker)(),
+                             void *context)
+{
+    if (node == NULL)
+    {
+        return false;
+    }
+
+#define LIST_WALK(l) cypher_expr_tree_walker((Node *) (l), walker, context)
+
+    if (IsA(node, ExtensibleNode))
+    {
+        /* Add our nodes that can appear post parsing stage */
+
+        ereport(ERROR,
+                (errmsg_internal("unrecognized ExtensibleNode: %s",
+                                 ((ExtensibleNode *)node)->extnodename)));
+    }
+#undef WALK
+#undef LIST_WALK
+    else
+    {
+        return expression_tree_walker(node, walker, context);
+    }
+
+    return false;
+}
+
 static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
                              const char *query_str, int query_loc,
                              char *graph_name, uint32 graph_oid, Param *params)
@@ -680,7 +917,9 @@ static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
         next->prev = clause;
 
         if (clause != NULL)
+        {
             clause->next = next;
+        }
         clause = next;
     }
 
@@ -711,6 +950,7 @@ static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
     cpstate->params = params;
     cpstate->default_alias_num = 0;
     cpstate->entities = NIL;
+    cpstate->subquery_where_flag = false;
     /*
      * install error context callback to adjust an error position since
      * locations in stmt are 0 based
@@ -741,7 +981,7 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
     Query *query;
     const bool lateral = false;
     Query *subquery;
-    RangeTblEntry *rte;
+    ParseNamespaceItem *pnsi;
     int rtindex;
     ListCell *lt;
     ListCell *lc1;
@@ -769,7 +1009,7 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
     pstate->p_lateral_active = false;
     pstate->p_expr_kind = EXPR_KIND_NONE;
 
-    // ALIAS Syntax makes `RESJUNK`. So, It must be skipping.
+    /* ALIAS Syntax makes `RESJUNK`. So, It must be skipping. */
     foreach(lt, subquery->targetList)
     {
         TargetEntry *te = lfirst(lt);
@@ -779,7 +1019,7 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
         }
     }
 
-    // check the number of attributes first
+    /* check the number of attributes first */
     if (attr_count != rtfunc->funccolcount)
     {
         ereport(ERROR,
@@ -788,17 +1028,25 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
                  parser_errposition(pstate, exprLocation(rtfunc->funcexpr))));
     }
 
-    rte = addRangeTableEntryForSubquery(pstate, subquery, makeAlias("_", NIL),
+    pnsi = addRangeTableEntryForSubquery(pstate, subquery, makeAlias("_", NIL),
                                         lateral, true);
-    rtindex = list_length(pstate->p_rtable);
-    Assert(rtindex == 1); // rte is the only RangeTblEntry in pstate
-    addRTEtoQuery(pstate, rte, true, true, true);
 
-    query->targetList = expandRelAttrs(pstate, rte, rtindex, 0, -1);
+    rtindex = list_length(pstate->p_rtable);
+    /* rte is the only RangeTblEntry in pstate */
+    if (rtindex !=1 )
+    {
+        ereport(ERROR,
+                (errcode(ERRCODE_DATATYPE_MISMATCH),
+                 errmsg("invalid value for rtindex")));
+    }
+
+
+    addNSItemToQuery(pstate, pnsi, true, true, true);
+    query->targetList = expandNSItemAttrs(pstate, pnsi, 0, true, -1);
 
     markTargetListOrigins(pstate, query->targetList);
 
-    // do the type coercion for each target entry
+    /* do the type coercion for each target entry */
     lc1 = list_head(rtfunc->funccolnames);
     lc2 = list_head(rtfunc->funccoltypes);
     lc3 = list_head(rtfunc->funccoltypmods);
@@ -807,15 +1055,20 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
         TargetEntry *te = lfirst(lt);
         Node *expr = (Node *)te->expr;
         Oid current_type;
+        int32 current_typmod;
         Oid target_type;
+        int32 target_typmod;
 
         Assert(!te->resjunk);
 
         current_type = exprType(expr);
+        current_typmod = exprTypmod(expr);
         target_type = lfirst_oid(lc2);
-        if (current_type != target_type)
+        target_typmod = lfirst_int(lc3);
+
+        if ((current_type != target_type) ||
+            (current_typmod != target_typmod))
         {
-            int32 target_typmod = lfirst_int(lc3);
             Node *new_expr;
 
             /*
@@ -844,9 +1097,9 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
             te->expr = (Expr *)new_expr;
         }
 
-        lc1 = lnext(lc1);
-        lc2 = lnext(lc2);
-        lc3 = lnext(lc3);
+        lc1 = lnext(rtfunc->funccolnames, lc1);
+        lc2 = lnext(rtfunc->funccoltypes, lc2);
+        lc3 = lnext(rtfunc->funccoltypmods, lc3);
     }
 
     query->rtable = pstate->p_rtable;
