@@ -44,13 +44,9 @@ int cypher_yylex(YYSTYPE *lvalp, YYLTYPE *llocp, ag_scanner_t scanner)
         DOT_DOT,
         TYPECAST,
         PLUS_EQ,
-        EQ_TILDE,
-        LEFT_CONTAINS,
-        RIGHT_CONTAINS,
-        ACCESS_PATH,
-        ANY_EXISTS,
-        ALL_EXISTS,
-        CONCAT
+        CHAR,
+        BQIDENT,
+        OP
     };
 
     ag_token token;
@@ -66,26 +62,27 @@ int cypher_yylex(YYSTYPE *lvalp, YYLTYPE *llocp, ag_scanner_t scanner)
         break;
     case AG_TOKEN_DECIMAL:
     case AG_TOKEN_STRING:
+    case AG_TOKEN_OP:
         lvalp->string = pstrdup(token.value.s);
         break;
     case AG_TOKEN_IDENTIFIER:
     {
-        const ScanKeyword *keyword;
+        int kwnum;
         char *ident;
 
-        keyword = ScanKeywordLookup(token.value.s, cypher_keywords,
-                                    num_cypher_keywords);
-        if (keyword)
+        kwnum = ScanKeywordLookup(token.value.s, &CypherKeyword);
+        if (kwnum >= 0)
         {
             /*
              * use token.value.s instead of keyword->name to preserve
              * case sensitivity
              */
+            lvalp->keyword = GetScanKeyword(kwnum, &CypherKeyword);
             ident = pstrdup(token.value.s);
             truncate_identifier(ident, strlen(ident), true);
             lvalp->string = ident;
             *llocp = token.location;
-            return keyword->value;
+            return CypherKeywordTokens[kwnum];
         }
 
         ident = pstrdup(token.value.s);
@@ -93,6 +90,18 @@ int cypher_yylex(YYSTYPE *lvalp, YYLTYPE *llocp, ag_scanner_t scanner)
         lvalp->string = ident;
         break;
     }
+    case AG_TOKEN_BQIDENT:
+    {
+        char *ident;
+
+        /* these are identifiers, just back ticked */
+        token.type = AG_TOKEN_IDENTIFIER;
+
+        ident = pstrdup(token.value.s);
+        truncate_identifier(ident, strlen(ident), true);
+        lvalp->string = ident;
+        break;
+    }
     case AG_TOKEN_PARAMETER:
         lvalp->string = pstrdup(token.value.s);
         break;
@@ -101,14 +110,6 @@ int cypher_yylex(YYSTYPE *lvalp, YYLTYPE *llocp, ag_scanner_t scanner)
     case AG_TOKEN_GT_EQ:
     case AG_TOKEN_DOT_DOT:
     case AG_TOKEN_PLUS_EQ:
-    case AG_TOKEN_EQ_TILDE:
-    case AG_TOKEN_ACCESS_PATH:
-    case AG_TOKEN_ALL_EXISTS:
-    case AG_TOKEN_ANY_EXISTS:
-    case AG_TOKEN_LEFT_CONTAINS:
-    case AG_TOKEN_RIGHT_CONTAINS:
-    case AG_TOKEN_CONCAT:
-        break;
     case AG_TOKEN_TYPECAST:
         break;
     case AG_TOKEN_CHAR:
