@@ -20,19 +20,15 @@
 #ifndef AG_CYPHER_CLAUSE_H
 #define AG_CYPHER_CLAUSE_H
 
-#include "nodes/parsenodes.h"
-#include "nodes/pg_list.h"
-#include "parser/parse_node.h"
-
 #include "parser/cypher_parse_node.h"
 
 typedef struct cypher_clause cypher_clause;
 
 struct cypher_clause
 {
-    cypher_clause *next; //next clause
+    cypher_clause *next; /* next clause */
     Node *self;
-    cypher_clause *prev; // previous clause
+    cypher_clause *prev; /* previous clause */
 };
 
 Query *transform_cypher_clause(cypher_parsestate *cpstate,
