@@ -20,9 +20,6 @@
 #ifndef AG_CYPHER_PARSE_NODE_H
 #define AG_CYPHER_PARSE_NODE_H
 
-#include "nodes/primnodes.h"
-#include "parser/parse_node.h"
-
 #include "nodes/cypher_nodes.h"
 
 /*
@@ -43,6 +40,7 @@ typedef struct cypher_parsestate
     int default_alias_num;
     List *entities;
     List *property_constraint_quals;
+    bool subquery_where_flag; /* flag for knowing we are in a subquery where */
     /*
      * To flag when an aggregate has been found in an expression during an
      * expression transform. This is used during the return_item list transform
@@ -57,8 +55,8 @@ typedef struct cypher_parsestate
 typedef struct errpos_ecb_state
 {
     ErrorContextCallback ecb;
-    ParseState *pstate; // ParseState of query that has subquery being parsed
-    int query_loc; // location of subquery starting from p_sourcetext
+    ParseState *pstate; /* ParseState of query that has subquery being parsed */
+    int query_loc; /* location of subquery starting from p_sourcetext */
 } errpos_ecb_state;
 
 cypher_parsestate *make_cypher_parsestate(cypher_parsestate *parent_cpstate);
@@ -68,7 +66,6 @@ void free_cypher_parsestate(cypher_parsestate *cpstate);
 void setup_errpos_ecb(errpos_ecb_state *ecb_state, ParseState *pstate,
                       int query_loc);
 void cancel_errpos_ecb(errpos_ecb_state *ecb_state);
-RangeTblEntry *find_rte(cypher_parsestate *cpstate, char *varname);
 char *get_next_default_alias(cypher_parsestate *cpstate);
 
 #endif
